import 'package:car_app/converter/appwrite_converters/user.dart';
import 'package:car_app/model/calendar_event.dart';

extension CalendarEventMapConverter on CalendarEvent {
  static const String idFieldName = "id";
  static const String teamIdFieldName = "teamId";
  static const String titleFieldName = "title";
  static const String descriptionFieldName = "description";
  static const String startFieldName = "start";
  static const String endFieldName = "end";
  static const String userFieldName = "user";

  Map<String, dynamic> toMap() {
    return {
      idFieldName: id,
      teamIdFieldName: teamId,
      titleFieldName: title,
      descriptionFieldName: description,
      startFieldName: start.toUtc().toIso8601String(),
      endFieldName: end.toUtc().toIso8601String(),
    };
  }

  static CalendarEvent<String> fromMap(Map<String, dynamic> map) {
    var id = map[idFieldName] as String;
    var teamId = map[teamIdFieldName] as String;
    var title = map[titleFieldName] as String;
    var description = map[descriptionFieldName] as String?;
    var start = DateTime.parse(map[startFieldName] as String).toLocal();
    var end = DateTime.parse(map[endFieldName] as String).toLocal();

    var decodedUser = map[userFieldName] as Map<String, dynamic>;
    var user = UserMapCoverter.fromMap(decodedUser);

    return CalendarEvent(
      id: id,
      user: user,
      teamId: teamId,
      title: title,
      description: description,
      start: start,
      end: end
    );
  }
}