import 'dart:convert';

import 'package:appwrite/appwrite.dart';
import 'package:car_app/converter/appwrite_converters/calendar_event.dart';
import 'package:car_app/model/calendar_event.dart';
import 'package:car_app/service/appwrite/appwrite.dart';
import 'package:car_app/service/interface/calendar_event.dart';

class AppwriteCalendarEventService extends CalendarEventService {
  static const String _calendarRoute = "/calendar";

  final AppwriteService _appwriteService = AppwriteService();
  final Functions _functions = AppwriteService().functions;
  final String _apiFunctionId = AppwriteService().apiFunctionId;

  @override
  Future<CalendarEvent> add(CalendarEvent event) async {
    var action = "add calendar event";
    var execution = await _appwriteService.executeFunction(
      _functions.createExecution(functionId: _apiFunctionId, body: jsonEncode(event.toMap()), method: "POST", path: _calendarRoute),
      action,
    );

    _appwriteService.throwIfExecutionBodyIsEmpty(execution, action);

    return _appwriteService.fromExecution(execution, CalendarEventMapConverter.fromMap, action);
  }

  @override
  Stream<CalendarEvent> getAll() async* {
    var action = "get all calendar events";
    var execution = await _appwriteService.executeFunction(
      _functions.createExecution(functionId: _apiFunctionId, method: "GET", body: "{}", path: _calendarRoute),
      action,
    );

    if(!_appwriteService.validateExecutionBody(execution)) {
      return;
    }

    yield* _appwriteService.streamFromExecution(execution, CalendarEventMapConverter.fromMap, action);
  }
  
  @override
  Future<CalendarEvent> update(CalendarEvent event) async {
    var action = "update calendar event";
    var execution = await _appwriteService.executeFunction(
      _functions.createExecution(functionId: _apiFunctionId, body: jsonEncode(event.toMap()), method: "PUT", path: _calendarRoute),
      action,
    );

    _appwriteService.throwIfExecutionBodyIsEmpty(execution, action);

    return _appwriteService.fromExecution(execution, CalendarEventMapConverter.fromMap, action);
  }

  @override
  Future<CalendarEvent> delete(CalendarEvent event) async {
    var action = "delete calendar event";
    var execution = await _appwriteService.executeFunction(
      _functions.createExecution(functionId: _apiFunctionId, body: jsonEncode(event.toMap()), method: "DELETE", path: _calendarRoute),
      action,
    );

    _appwriteService.throwIfExecutionBodyIsEmpty(execution, action);

    return _appwriteService.fromExecution(execution, CalendarEventMapConverter.fromMap, action);
  }
}
